@@ -461,69 +461,14 @@ HRESULT VLCPlugin::onLoad(void)
 
 void VLCPlugin::initVLC()
 {
-    extern HMODULE DllGetModule();
+    static const char * const ppsz_argv[] = {
+        "-vv",
+        "--no-stats",
+        "--intf=dummy",
+        "--no-video-title-show",
+    };
 
-    /*
-    ** default initialization options
-    */
-    const char *ppsz_argv[32] = { };
-    int   ppsz_argc = 0;
-
-    char p_progpath[MAX_PATH];
-    {
-        TCHAR w_progpath[MAX_PATH];
-        DWORD len = GetModuleFileName(DllGetModule(), w_progpath, MAX_PATH);
-        w_progpath[MAX_PATH-1] = '\0';
-        if( len > 0 )
-        {
-            len = WideCharToMultiByte(CP_UTF8, 0, w_progpath, len, p_progpath,
-                       sizeof(p_progpath)-1, NULL, NULL);
-            if( len > 0 )
-            {
-                p_progpath[len] = '\0';
-                ppsz_argv[0] = p_progpath;
-            }
-        }
-    }
-
-    ppsz_argv[ppsz_argc++] = "-vv";
-
-    HKEY h_key;
-    char p_pluginpath[MAX_PATH];
-    if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("Software\\VideoLAN\\VLC"),
-                      0, KEY_READ, &h_key ) == ERROR_SUCCESS )
-    {
-        DWORD i_type, i_data = MAX_PATH;
-        TCHAR w_pluginpath[MAX_PATH];
-        if( RegQueryValueEx( h_key, TEXT("InstallDir"), 0, &i_type,
-                             (LPBYTE)w_pluginpath, &i_data ) == ERROR_SUCCESS )
-        {
-            w_pluginpath[MAX_PATH-1] = '\0';
-            if( i_type == REG_SZ )
-            {
-                if( WideCharToMultiByte(CP_UTF8, 0, w_pluginpath, -1, p_pluginpath,
-                         sizeof(p_pluginpath)-sizeof("\\plugins")+1, NULL, NULL) )
-                {
-                    strcat( p_pluginpath, "\\plugins" );
-                    ppsz_argv[ppsz_argc++] = "--plugin-path";
-                    ppsz_argv[ppsz_argc++] = p_pluginpath;
-                }
-            }
-        }
-        RegCloseKey( h_key );
-    }
-
-    // make sure plugin isn't affected with VLC single instance mode
-    ppsz_argv[ppsz_argc++] = "--no-one-instance";
-
-    /* common settings */
-    ppsz_argv[ppsz_argc++] = "-vv";
-    ppsz_argv[ppsz_argc++] = "--no-stats";
-    ppsz_argv[ppsz_argc++] = "--no-media-library";
-    ppsz_argv[ppsz_argc++] = "--intf=dummy";
-    ppsz_argv[ppsz_argc++] = "--no-video-title-show";
-
-    _p_libvlc = libvlc_new(ppsz_argc, ppsz_argv);
+    _p_libvlc = libvlc_new(sizeof(ppsz_argv) / sizeof(*ppsz_argv), ppsz_argv);
     if( !_p_libvlc )
         return;
 
