@@ -11,7 +11,6 @@
  *          Pierre d'Herbemont <pdherbemont # videolan.org>
  *          David Fuhrmann <david dot fuhrmann at googlemail dot com>
  *
- *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -34,18 +33,21 @@
 #include <QuartzCore/QuartzCore.h>
 #include <AppKit/AppKit.h>
 
-@interface VLCNoMediaLayer : CALayer
+#define SHOW_BRANDING 1
 
-@end
-
-@interface VLCPlaybackLayer : CALayer {
-    CGColorSpaceRef _colorspace;
+@interface VLCNoMediaLayer : CALayer {
     VlcPluginMac *_cppPlugin;
 }
 @property (readwrite) VlcPluginMac * cppPlugin;
 
 @end
 
+@interface VLCBrowserRootLayer : CALayer
+- (void)addVoutLayer:(CALayer *)aLayer;
+- (void)removeVoutLayer:(CALayer *)aLayer;
+- (CGSize)currentOutputSize;
+@end
+
 @interface VLCControllerLayer : CALayer {
     CGImageRef _playImage;
     CGImageRef _pauseImage;
@@ -110,8 +112,8 @@
 - (CGDirectDisplayID)displayID;
 @end
 
-static CALayer * browserRootLayer;
-static VLCPlaybackLayer * playbackLayer;
+static VLCBrowserRootLayer * browserRootLayer;
+static CALayer * playbackLayer;
 static VLCNoMediaLayer * noMediaLayer;
 static VLCControllerLayer * controllerLayer;
 static VLCFullscreenWindow * fullscreenWindow;
@@ -120,7 +122,7 @@ static VLCFullscreenContentView * fullscreenView;
 VlcPluginMac::VlcPluginMac(NPP instance, NPuint16_t mode) :
     VlcPluginBase(instance, mode)
 {
-    browserRootLayer = [[CALayer alloc] init];
+    browserRootLayer = [[VLCBrowserRootLayer alloc] init];
 }
 
 VlcPluginMac::~VlcPluginMac()
@@ -134,73 +136,8 @@ VlcPluginMac::~VlcPluginMac()
 
 void VlcPluginMac::set_player_window()
 {
-    libvlc_video_set_format_callbacks(getMD(),
-                                      video_format_proxy,
-                                      video_cleanup_proxy);
-    libvlc_video_set_callbacks(getMD(),
-                               video_lock_proxy,
-                               video_unlock_proxy,
-                               video_display_proxy,
-                               this);
-}
-
-unsigned VlcPluginMac::video_format_cb(char *chroma,
-                                       unsigned *width, unsigned *height,
-                                       unsigned *pitches, unsigned *lines)
-{
-    if ( p_browser ) {
-        /* request video in fullscreen size. scaling will be done by CA */
-        NSSize screenSize = [[NSScreen mainScreen] visibleFrame].size;
-        float src_aspect = (float)(*width) / (*height);
-        float dst_aspect = (float)screenSize.width/screenSize.height;
-        if ( src_aspect > dst_aspect ) {
-            if( screenSize.width != (*width) ) { //don't scale if size equal
-                (*width) = screenSize.width;
-                (*height) = static_cast<unsigned>( (*width) / src_aspect + 0.5);
-            }
-        } else {
-            if( screenSize.height != (*height) ) { //don't scale if size equal
-                (*height) = screenSize.height;
-                (*width) = static_cast<unsigned>( (*height) * src_aspect + 0.5);
-            }
-        }
-    }
-
-    m_media_width = (*width);
-    m_media_height = (*height);
-
-    memcpy(chroma, "RGBA", sizeof("RGBA")-1);
-    (*pitches) = m_media_width * 4;
-    (*lines) = m_media_height;
-
-    //+1 for vlc 2.0.3/2.1 bug workaround.
-    //They writes after buffer end boundary by some reason unknown to me...
-    m_frame_buf.resize( (*pitches) * ((*lines)+1) );
-
-    return 1;
-}
-
-void VlcPluginMac::video_cleanup_cb()
-{
-    m_frame_buf.resize(0);
-    m_media_width = 0;
-    m_media_height = 0;
-    [fullscreenWindow orderOut: nil];
-}
-
-void* VlcPluginMac::video_lock_cb(void **planes)
-{
-    (*planes) = m_frame_buf.empty()? 0 : &m_frame_buf[0];
-    return 0;
-}
-
-void VlcPluginMac::video_unlock_cb(void* /*picture*/, void *const * /*planes*/)
-{
-}
-
-void VlcPluginMac::video_display_cb(void * /*picture*/)
-{
-    [playbackLayer performSelectorOnMainThread:@selector(setNeedsDisplay) withObject: nil waitUntilDone:NO];
+    /* pass base layer to libvlc to pass it on to the vout */
+    libvlc_media_player_set_nsobject(getMD(), browserRootLayer);
 }
 
 void VlcPluginMac::toggle_fullscreen()
@@ -212,7 +149,7 @@ void VlcPluginMac::toggle_fullscreen()
 
     if (get_fullscreen() == 0) {
         if (!fullscreenWindow) {
-            /* this window is kind of useless. however, we need to support 10.5, since enterFullScreenMode depends on the 
+            /* this window is kind of useless. however, we need to support 10.5, since enterFullScreenMode depends on the
              * existance of a parent window. This is solved in 10.6 and we should remove the window once we require it. */
             fullscreenWindow = [[VLCFullscreenWindow alloc] initWithContentRect: NSMakeRect(npwindow.x, npwindow.y, npwindow.width, npwindow.height)];
             [fullscreenWindow setLevel: CGShieldingWindowLevel()];
@@ -279,9 +216,11 @@ void VlcPluginMac::update_controls()
     [controllerLayer setIsPlaying: playlist_isplaying()];
     [controllerLayer setIsFullscreen:this->get_fullscreen()];
 
-    if (player_has_vout()) {
+    libvlc_state_t currentstate = libvlc_media_player_get_state(getMD());
+    if (currentstate == libvlc_Playing || currentstate == libvlc_Paused || currentstate == libvlc_Opening) {
         [noMediaLayer setHidden: YES];
         [playbackLayer setHidden: NO];
+        [playbackLayer setNeedsDisplay];
     } else {
         [noMediaLayer setHidden: NO];
         [playbackLayer setHidden: YES];
@@ -310,14 +249,9 @@ NPError VlcPluginMac::get_root_layer(void *value)
 {
     noMediaLayer = [[VLCNoMediaLayer alloc] init];
     noMediaLayer.opaque = 1.;
+    [noMediaLayer setCppPlugin: this];
     [browserRootLayer addSublayer: noMediaLayer];
 
-    playbackLayer = [[VLCPlaybackLayer alloc] init];
-    playbackLayer.opaque = 1.;
-    [browserRootLayer addSublayer: playbackLayer];
-    [playbackLayer setCppPlugin: this];
-    [playbackLayer setHidden: YES];
-
     controllerLayer = [[VLCControllerLayer alloc] init];
     controllerLayer.opaque = 1.;
     [browserRootLayer addSublayer: controllerLayer];
@@ -436,110 +370,54 @@ bool VlcPluginMac::handle_event(void *event)
     return VlcPluginBase::handle_event(event);
 }
 
-@implementation VLCPlaybackLayer
-@synthesize cppPlugin = _cppPlugin;
+@implementation VLCBrowserRootLayer
 
 - (id)init
 {
     if (self = [super init]) {
         self.needsDisplayOnBoundsChange = YES;
         self.autoresizingMask = kCALayerWidthSizable | kCALayerHeightSizable;
-
-        _colorspace = CGColorSpaceCreateDeviceRGB();
     }
 
     return self;
 }
 
-- (void)dealloc
+- (void)addVoutLayer:(CALayer *)aLayer
 {
-    CGColorSpaceRelease(_colorspace);
-    [super dealloc];
+    [CATransaction begin];
+    playbackLayer = [aLayer retain];
+    playbackLayer.opaque = 1.;
+    playbackLayer.hidden = NO;
+    playbackLayer.bounds = noMediaLayer.bounds;
+    [self insertSublayer:playbackLayer below:controllerLayer];
+    [self setNeedsDisplay];
+    [playbackLayer setNeedsDisplay];
+    CGRect frame = playbackLayer.bounds;
+    frame.origin.x = 0.;
+    frame.origin.y = 0.;
+    playbackLayer.frame = frame;
+    [CATransaction commit];
 }
 
-- (void)drawInContext:(CGContextRef)cgContext
+- (void)removeVoutLayer:(CALayer *)aLayer
 {
-    if (!cgContext)
-        return;
+    [CATransaction begin];
+    [aLayer removeFromSuperlayer];
+    [CATransaction commit];
 
-    if (![self cppPlugin]->playlist_isplaying() || ![self cppPlugin]->player_has_vout())
-        return;
+    if (playbackLayer == aLayer)
+        [playbackLayer release];
+}
 
-    float media_width = (float)[self cppPlugin]->m_media_width;
-    float media_height = (float)[self cppPlugin]->m_media_height;
-
-    if (media_width == 0. || media_height == 0.)
-        return;
-
-    CGRect layerRect = self.bounds;
-    float display_width = 0.;
-    float display_height = 0.;
-
-    float src_aspect = (float)media_width / media_height;
-    float dst_aspect = (float)layerRect.size.width/layerRect.size.height;
-    if ( src_aspect > dst_aspect ) {
-        if( layerRect.size.width != media_width ) { //don't scale if size equal
-            display_width = layerRect.size.width;
-            display_height = display_width / src_aspect; // + 0.5);
-        } else {
-            display_width = media_width;
-            display_height = media_height;
-        }
-    } else {
-        if( layerRect.size.height != media_height ) { //don't scale if size equal
-            display_height = layerRect.size.height;
-            display_width = display_height * src_aspect; // + 0.5);
-        } else {
-            display_width = media_width;
-            display_height = media_height;
-        }
-    }
-
-    /* Compute the position of the video */
-    float left = (layerRect.size.width  - display_width)  / 2.;
-    float top  = (layerRect.size.height - display_height) / 2.;
-
-    CGContextSaveGState(cgContext);
-
-    static const size_t kComponentsPerPixel = 4;
-    static const size_t kBitsPerComponent = sizeof(unsigned char) * 8;
-
-    /* render frame */
-    CFDataRef dataRef = CFDataCreateWithBytesNoCopy(kCFAllocatorDefault,
-                                                    (const uint8_t *)&[self cppPlugin]->m_frame_buf[0],
-                                                    sizeof([self cppPlugin]->m_frame_buf[0]),
-                                                    kCFAllocatorNull);
-    CGDataProviderRef dataProvider = CGDataProviderCreateWithCFData(dataRef);
-
-    CGImageRef image = CGImageCreate(media_width,
-                                     media_height,
-                                     kBitsPerComponent,
-                                     kBitsPerComponent * kComponentsPerPixel,
-                                     kComponentsPerPixel * media_width,
-                                     _colorspace,
-                                     kCGBitmapByteOrder16Big,
-                                     dataProvider,
-                                     NULL,
-                                     true,
-                                     kCGRenderingIntentPerceptual);
-    if (!image) {
-        CGImageRelease(image);
-        CGDataProviderRelease(dataProvider);
-        CGContextRestoreGState(cgContext);
-        return;
-    }
-    CGRect rect = CGRectMake(left, top, display_width, display_height);
-    CGContextDrawImage(cgContext, rect, image);
-
-    CGImageRelease(image);
-    CGDataProviderRelease(dataProvider);
-
-    CGContextRestoreGState(cgContext);
+- (CGSize)currentOutputSize
+{
+    return [browserRootLayer visibleRect].size;
 }
 
 @end
 
 @implementation VLCNoMediaLayer
+@synthesize cppPlugin = _cppPlugin;
 
 - (id)init
 {
@@ -558,6 +436,7 @@ bool VlcPluginMac::handle_event(void *event)
 
     CGContextSaveGState(cgContext);
 
+#if SHOW_BRANDING
     // draw a gray background
     CGContextAddRect(cgContext, CGRectMake(0, 0, windowWidth, windowHeight));
     CGContextSetGrayFillColor(cgContext, .5, 1.);
@@ -612,6 +491,47 @@ bool VlcPluginMac::handle_event(void *event)
     CFRelease(textLine);
     CFRelease(attRef);
     CFRelease(stylesDict);
+#else
+    // draw a black rect
+    CGRect rect;
+    float media_width = [self cppPlugin]->m_media_width;
+    float media_height = [self cppPlugin]->m_media_height;
+
+    if (media_width == 0. || media_height == 0.)
+        CGRectMake(0, 0, windowWidth, windowHeight);
+    else {
+        CGRect layerRect = self.bounds;
+        float display_width = 0.;
+        float display_height = 0.;
+        float src_aspect = (float)media_width / media_height;
+        float dst_aspect = (float)layerRect.size.width/layerRect.size.height;
+        if ( src_aspect > dst_aspect ) {
+            if( layerRect.size.width != media_width ) { //don't scale if size equal
+                display_width = layerRect.size.width;
+                display_height = display_width / src_aspect; // + 0.5);
+            } else {
+                display_width = media_width;
+                display_height = media_height;
+            }
+        } else {
+            if( layerRect.size.height != media_height ) { //don't scale if size equal
+                display_height = layerRect.size.height;
+                display_width = display_height * src_aspect; // + 0.5);
+            } else {
+                display_width = media_width;
+                display_height = media_height;
+            }
+        }
+
+        float left = (layerRect.size.width  - display_width)  / 2.;
+        float top  = (layerRect.size.height - display_height) / 2.;
+        CGRect rect = CGRectMake(left, top, display_width, display_height);
+    }
+
+    CGContextAddRect(cgContext, rect);
+    CGContextSetGrayFillColor(cgContext, 0., 1.);
+    CGContextDrawPath(cgContext, kCGPathFill);
+#endif
 
     CGContextRestoreGState(cgContext);
 }
